const WebSocket = require('ws');
const tunnel = require('tunnel');

function connectToWebSocketWithProxy(endpoint, proxyUrl) {
  if(!proxyUrl || (!proxyUrl.startsWith('http://') && !proxyUrl.startsWith('https://'))) {
    return;
  }
  const agent = createProxyAgent(proxyUrl);
  if(!agent) {
    return;
  }
  return new WebSocket(endpoint, {
    agent: agent
  });
}

function createProxyAgent(proxyUrl) {
  if(!proxyUrl) return;
  try {
    const {
      hostname,
      port,
      username,
      password
    } = parseProxyUrl(proxyUrl);
    const proxyOptions = {
      proxy: {
        host: hostname,
        port: port
      },
      headers: {}
    };
    if(username && password) {
      proxyOptions.proxy.proxyAuth = `${username}:${password}`;
    }
    return tunnel.httpsOverHttp(proxyOptions);
  }
  catch (error) {
    return;
  }
}

function parseProxyUrl(proxyUrl) {
  const parsedUrl = new URL(proxyUrl);
  return {
    hostname: parsedUrl.hostname,
    port: parsedUrl.port || 3128,
    username: parsedUrl.username || '',
    password: parsedUrl.password || ''
  };
}
module.exports = connectToWebSocketWithProxy;